# elm-music-theory

## A toolkit for musical ideas

This package allows you to work with concepts in tonal music like pitches, intervals, chords, keys, and scales. You can use it to analyze music, generate musical structures, and solve musical problems like [harmonization](https://en.wikipedia.org/wiki/Harmonization) and [counterpoint](https://en.wikipedia.org/wiki/Counterpoint).

It is built with these goals in mind:

1. Model musical concepts and their interactions accurately and coherently.
2. Use ideas and processes that are familiar to practicing musicians.
3. Apply broadly across musical idioms.

## Examples

Source code for these examples can be found in the `/examples` directory of this repo:

- **"Lead Sheet"** ([View live example](https://duncanmalashock.github.io/elm-music-theory-examples/lead-sheet/)) - Convert a chord progression into Roman numeral analysis, and transpose it into any key. 
- **"Chord Voicings"**  ([View live example](https://duncanmalashock.github.io/elm-music-theory-examples/chord-voicings/)) - Choose a chord and a method for voicing it. See it notated, and hear the result.  

## Learning resources

**New to music theory?** I can recommend these resources:

### Free, online

- The [Gitbook](https://duncanmalashock.gitbook.io/music-theory/) for this project, which attempts to explain the domain concepts in this package and how they are modeled
- [Fundamentals of Music Theory](https://www.coursera.org/learn/edinburgh-music-theory/), a free course on [Coursera](https://www.coursera.org/)
- [Open Music Theory](http://openmusictheory.com/), a free online textbook from [Hybrid Pedagogy](https://hybridpedagogy.org/)

### Books

- [The Jazz Theory Book](https://www.worldcat.org/title/jazz-theory-book/oclc/981421631) by Mark Levine
- [Jazz Arranging Techniques](http://lindsayjazz.com/jazz-arranging-techniques/) by Gary Lindsay
- [Counterpoint](https://www.worldcat.org/title/counterpoint-based-on-eighteenth-century-practice/oclc/726187199) by Kent Kennan

## Contributing

**Interested in contributing to this project?** Please take a look at the [contributing guidelines](https://github.com/duncanmalashock/elm-music-theory/blob/master/CONTRIBUTING.md) for this repo for ways to do so. 

## Special thanks

Thanks to Leif Battermann ([@battermann](https://github.com/battermann)) for collaborating with me on early versions of some of the designs that are central to this package.  

Thanks to my friend and colleague Matthew Griffith ([@mdgriffith](https://github.com/mdgriffith)) for his advice and encouragement. This package was designed better (and released sooner) because of him.

